Magic-number elimination.
[open-adventure.git] / actions.c
index 2ad4623dce25e693b44a01157d212629b2566731..06da68805481e1526c7a24687b09c9224188b6b1 100644 (file)
--- a/actions.c
+++ b/actions.c
@@ -7,6 +7,7 @@
 static int fill(token_t, token_t);
 
 static void state_change(long obj, long state)
+/* Object must have a change-message list for this to be useful; only some do */
 {
     game.prop[obj] = state;
     pspeak(obj, change, state, true);
@@ -260,7 +261,7 @@ static int vbreak(token_t verb, token_t obj)
         if (TOTING(VASE))
             drop(VASE, game.loc);
         state_change(VASE, VASE_BROKEN);
-        game.fixed[VASE] = -1;
+        game.fixed[VASE] = IS_FIXED;
         return GO_CLEAROBJ;
     }
     rspeak(actions[verb].message);
@@ -301,8 +302,9 @@ static int vcarry(token_t verb, token_t obj)
         return GO_CLEAROBJ;
     }
 
-    if (game.fixed[obj] != 0) {
-        if (obj == PLANT && game.prop[PLANT] <= 0) {
+    if (game.fixed[obj] != IS_FREE) {
+       /* Next guard tests whether plant is tiny or stashed */
+        if (obj == PLANT && game.prop[PLANT] <= PLANT_THIRSTY) {
             rspeak(DEEP_ROOTS);
             return GO_CLEAROBJ;
         }
@@ -361,7 +363,7 @@ static int vcarry(token_t verb, token_t obj)
 
     }
 
-    if (obj == BIRD && game.prop[BIRD] != BIRD_CAGED && -1 - game.prop[BIRD] != BIRD_CAGED) {
+    if (obj == BIRD && game.prop[BIRD] != BIRD_CAGED && STASHED(BIRD) != BIRD_CAGED) {
         if (game.prop[BIRD] == BIRD_FOREST_UNCAGED) {
             DESTROY(BIRD);
             rspeak(BIRD_CRAP);
@@ -377,18 +379,16 @@ static int vcarry(token_t verb, token_t obj)
         }
         game.prop[BIRD] = BIRD_CAGED;
     }
-    /* FIXME: Arithmetic on state numbers */
     if ((obj == BIRD ||
          obj == CAGE) &&
-        (game.prop[BIRD] == BIRD_CAGED ||
-         -1 - game.prop[BIRD] == 1))
+        (game.prop[BIRD] == BIRD_CAGED || STASHED(BIRD) == BIRD_CAGED))
+        /* expression maps BIRD to CAGE and CAGE to BIRD */
         carry(BIRD + CAGE - obj, game.loc);
     carry(obj, game.loc);
     if (obj == BOTTLE && LIQUID() != NO_OBJECT)
         game.place[LIQUID()] = CARRIED;
-    if (GSTONE(obj) && game.prop[obj] != 0) {
-        game.prop[obj]
-            = STATE_GROUND;
+    if (GSTONE(obj) && game.prop[obj] != STATE_FOUND) {
+        game.prop[obj] = STATE_FOUND;
         game.prop[CAVITY] = CAVITY_EMPTY;
     }
     rspeak(OK_MAN);
@@ -408,16 +408,16 @@ static int chain(token_t verb)
             return GO_CLEAROBJ;
         }
         game.prop[CHAIN] = CHAIN_HEAP;
-        game.fixed[CHAIN] = CHAIN_HEAP;
+        game.fixed[CHAIN] = IS_FREE;
         if (game.prop[BEAR] != BEAR_DEAD)
             game.prop[BEAR] = CONTENTED_BEAR;
 
         switch (game.prop[BEAR]) {
         case BEAR_DEAD:
-            game.fixed[BEAR] = -1;
+            game.fixed[BEAR] = IS_FIXED;
             break;
         default:
-            game.fixed[BEAR] = 0;
+            game.fixed[BEAR] = IS_FREE;
         }
         rspeak(CHAIN_UNLOCKED);
         return GO_CLEAROBJ;
@@ -436,7 +436,7 @@ static int chain(token_t verb)
 
     if (TOTING(CHAIN))
         drop(CHAIN, game.loc);
-    game.fixed[CHAIN] = -1;
+    game.fixed[CHAIN] = IS_FIXED;
 
     rspeak(CHAIN_LOCKED);
     return GO_CLEAROBJ;
@@ -447,12 +447,11 @@ static int discard(token_t verb, token_t obj, bool just_do_it)
  *  bird (might attack snake or dragon) and cage (might contain bird) and vase.
  *  Drop coins at vending machine for extra batteries. */
 {
-    int spk = actions[verb].message;
     if (!just_do_it) {
         if (TOTING(ROD2) && obj == ROD && !TOTING(ROD))
             obj = ROD2;
         if (!TOTING(obj)) {
-            rspeak(spk);
+            rspeak(actions[verb].message);
             return GO_CLEAROBJ;
         }
         if (obj == BIRD && HERE(SNAKE)) {
@@ -465,11 +464,11 @@ static int discard(token_t verb, token_t obj, bool just_do_it)
 
         } else if ((GSTONE(obj) && AT(CAVITY) && game.prop[CAVITY] != CAVITY_FULL)) {
             rspeak(GEM_FITS);
-            game.prop[obj] = 1;
+            game.prop[obj] = STATE_IN_CAVITY;
             game.prop[CAVITY] = CAVITY_FULL;
             if (HERE(RUG) && ((obj == EMERALD && game.prop[RUG] != RUG_HOVER) ||
                               (obj == RUBY && game.prop[RUG] == RUG_HOVER))) {
-                spk = RUG_RISES;
+                int spk = RUG_RISES;
                 if (TOTING(RUG))
                     spk = RUG_WIGGLES;
                 if (obj == RUBY)
@@ -479,7 +478,7 @@ static int discard(token_t verb, token_t obj, bool just_do_it)
                     /* FIXME: Arithmetic on state numbers */
                     int k = 2 - game.prop[RUG];
                     game.prop[RUG] = k;
-                    if (k == 2)
+                    if (k == RUG_HOVER)
                         k = objects[SAPPH].plac;
                     move(RUG + NOBJECTS, k);
                 }
@@ -504,18 +503,17 @@ static int discard(token_t verb, token_t obj, bool just_do_it)
                    game.loc == objects[PILLOW].plac) {
             rspeak(OK_MAN);
         } else {
-            game.prop[VASE] = VASE_BROKEN;
-            if (AT(PILLOW))
-                game.prop[VASE] = VASE_WHOLE;
-            pspeak(VASE, look, game.prop[VASE] + 1, true);
+           state_change(VASE, AT(PILLOW)
+                        ? VASE_WHOLE
+                        : VASE_DROPPED);
             if (game.prop[VASE] != VASE_WHOLE)
-                game.fixed[VASE] = -1;
+                game.fixed[VASE] = IS_FIXED;
         }
     }
     int k = LIQUID();
     if (k == obj)
         obj = BOTTLE;
-    if (obj == BOTTLE && k != 0)
+    if (obj == BOTTLE && k != NO_OBJECT)
         game.place[k] = LOC_NOWHERE;
     if (obj == CAGE && game.prop[BIRD] == BIRD_CAGED)
         drop(BIRD, game.loc);
@@ -549,9 +547,8 @@ static int drink(token_t verb, token_t obj)
         return GO_CLEAROBJ;
     }
     if (LIQUID() == WATER && HERE(BOTTLE)) {
-        game.prop[BOTTLE] = EMPTY_BOTTLE;
         game.place[WATER] = LOC_NOWHERE;
-        rspeak(BOTTLE_EMPTY);
+        state_change(BOTTLE, EMPTY_BOTTLE);
         return GO_CLEAROBJ;
     }
 
@@ -663,8 +660,8 @@ static int feed(token_t verb, token_t obj)
         if (HERE(FOOD)) {
             DESTROY(FOOD);
             game.prop[BEAR] = SITTING_BEAR;
-            game.fixed[AXE] = 0;
-            game.prop[AXE] = 0;
+            game.fixed[AXE] = IS_FREE;
+            game.prop[AXE] = AXE_HERE;
             spk = BEAR_TAMED;
         }
     } else if (obj == OGRE) {
@@ -692,8 +689,8 @@ int fill(token_t verb, token_t obj)
         }
         rspeak(SHATTER_VASE);
         game.prop[VASE] = VASE_BROKEN;
-        game.fixed[VASE] = -1;
-        return (discard(verb, obj, true));
+        game.fixed[VASE] = IS_FIXED;
+        return (discard(verb, VASE, true));
     }
 
     if (obj == URN) {
@@ -744,13 +741,11 @@ int fill(token_t verb, token_t obj)
         return GO_CLEAROBJ;
     }
 
-    game.prop[BOTTLE] = (LIQLOC(game.loc) == OIL) ? OIL_BOTTLE : WATER_BOTTLE;
+    state_change(BOTTLE, (LIQLOC(game.loc) == OIL)
+                ? OIL_BOTTLE
+                : WATER_BOTTLE);
     if (TOTING(BOTTLE))
         game.place[LIQUID()] = CARRIED;
-    if (LIQUID() == OIL)
-        rspeak(BOTTLED_OIL);
-    else
-        rspeak(BOTTLED_WATER);
     return GO_CLEAROBJ;
 }
 
@@ -808,10 +803,10 @@ static int fly(token_t verb, token_t obj)
     /* FIXME: Arithmetic on location values */
     game.newloc = game.place[RUG] + game.fixed[RUG] - game.loc;
 
-    if (game.prop[SAPPH] >= 0) {
-        rspeak(RUG_RETURNS);
-    } else {
+    if (game.prop[SAPPH] == STATE_NOTFOUND) {
         rspeak(RUG_GOES);
+    } else {
+        rspeak(RUG_RETURNS);
     }
     return GO_TERMINATE;
 }
@@ -874,7 +869,7 @@ static int light(token_t verb, token_t obj)
 }
 
 static int listen(void)
/*  Listen.  Intransitive only.  Print stuff based on objsnd/locsnd. */
 {
     long sound = locations[game.loc].sound;
     if (sound != SILENT) {
@@ -894,8 +889,7 @@ static int listen(void)
         long packed_zzword = token_to_packed(game.zzword);
         pspeak(i, hear, mi, true, packed_zzword);
         rspeak(NO_MESSAGE);
-        /* FIXME: Magic number, sensitive to bird state logic */
-        if (i == BIRD && game.prop[i] == 5)
+        if (i == BIRD && mi == BIRD_ENDSTATE)
             DESTROY(BIRD);
         return GO_CLEAROBJ;
     }
@@ -1112,7 +1106,6 @@ static int say(struct command_t *command)
     char word1[TOKLEN + 1];
     packed_to_token(command->wd1, word1);
     int wd = (int) get_vocab_id(word1);
-    /* FIXME: magic numbers */
     if (wd == MOTION_WORD(XYZZY) ||
         wd == MOTION_WORD(PLUGH) ||
         wd == MOTION_WORD(PLOVER) ||
@@ -1139,8 +1132,6 @@ static int throw (struct command_t *command)
  *  (Only way to do so!)  Axe also special for dragon, bear, and
  *  troll.  Treasures special for troll. */
 {
-    if (TOTING(ROD2) && command->obj == ROD && !TOTING(ROD))
-        command->obj = ROD2;
     if (!TOTING(command->obj)) {
         rspeak(actions[command->verb].message);
         return GO_CLEAROBJ;
@@ -1174,7 +1165,7 @@ static int throw (struct command_t *command)
             else if (HERE(BEAR) && game.prop[BEAR] == UNTAMED_BEAR) {
                 /* This'll teach him to throw the axe at the bear! */
                 drop(AXE, game.loc);
-                game.fixed[AXE] = -1;
+                game.fixed[AXE] = IS_FIXED;
                 juggle(BEAR);
                 state_change(AXE, AXE_LOST);
                 return GO_CLEAROBJ;
@@ -1226,7 +1217,7 @@ static int wave(token_t verb, token_t obj)
 
     if (game.prop[BIRD] == BIRD_UNCAGED && game.loc == game.place[STEPS] && game.prop[JADE] < 0) {
         drop(JADE, game.loc);
-        game.prop[JADE] = 0;
+        game.prop[JADE] = STATE_FOUND;
         --game.tally;
         rspeak(NECKLACE_FLY);
         return GO_CLEAROBJ;
@@ -874,7 +869,7 @@ static int light(token_t verb, token_t obj)
 
 static int listen(void)
-/*  Listen.  Intransitive only.  Print stuff based on objsnd/locsnd. */
+/*  Listen.  Intransitive only.  Print stuff based on object sound proprties. */
 {
     long sound = locations[game.loc].sound;